#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Indicator;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Strategy;
#endregion

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    /// <summary>
    /// Enter the description of your strategy here
    /// </summary>
    [Description("Enter the description of your strategy here")]
    public class SoybeanSeasonalChannelBreakout : Strategy
    {
        #region Variables
		// Inputs
        private int 	length		= 4; // Default setting for Length
        private int 	d1DXY		= 25; // Default setting for D1DXY
        private int 	d2SNL		= 12; // Default setting for D2SNL
        private int 	sellMonth	= 6; // Default setting for SellMonth
        private int 	buyMonth	= 9; // Default setting for BuyMonth
        private double	lRSNLSell	= -0.8; // Default setting for LRSNLSell
        private double	lRSDXYSell	= 0.3; // Default setting for LRSDXYSell
		private int		mASNL		= 15;
		private int		mASell		= 15;
		private int		mADXY		= 50;
		
		// Variables
		private double	lRSNL		= 0;
		private double	lRSDXY		= 0;
		private bool	seasonalBuy	= false;
		private bool	seasonalShort = false;
		
		// DataSeries
		private DataSeries rS;
		private DataSeries rSTemp;
        #endregion

        /// <summary>
        /// This method is used to configure the strategy and is called once before any strategy method is called.
        /// </summary>
        protected override void Initialize()
        {
            CalculateOnBarClose = true;
			Add("DXY", PeriodType.Day, 1);
			Add("SNL", PeriodType.Day, 1);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			// Sync DataSeries to correct BarsInProgress
			if (rSTemp == null)
				rSTemp = new DataSeries(SMA(BarsArray[2], 2));
			
			if (rS == null)
				rS = new DataSeries(SMA(BarsArray[2], 2));
			
			// Process logic after all bar's have been evaluated for the day
			if (BarsInProgress == 2)
			{
				if (CurrentBar > 3)
				{
					rSTemp.Set(Closes[0][0] / Closes[2][0]);
					rS.Set(EMA(rSTemp, 3)[0]);
				}
				
				if (CurrentBar > D2SNL + 3)
					lRSNL = (LinReg(rS, D2SNL)[0] - LinReg(rS, D2SNL)[D2SNL]) / LinReg(rS, D2SNL)[D2SNL] / D2SNL * 100;
				
				if (CurrentBar > D1DXY)
					lRSDXY = (LinReg(Closes[1], D1DXY)[0] - LinReg(Closes[1], D1DXY)[D1DXY]) / LinReg(Closes[1], D1DXY)[D1DXY] / D1DXY * 100;
			
				if (Time[0].Month < SellMonth || Time[0].Month > BuyMonth)
				{
					seasonalBuy = true;
					seasonalShort = false;
				}
				else if (Time[0].Month > SellMonth && Time[0].Month < (BuyMonth + 1))
				{
					seasonalBuy = false;
					seasonalShort = true;
				}
				else
				{
					seasonalBuy = false;
					seasonalShort = false;
				}
				
				// Entry Conditions for Longs
				if (seasonalBuy == true && lRSDXY < lRSDXYSell && Closes[0][0] > MAX(Closes[0], Length)[1] + 2 && lRSNL > lRSNLSell)
					EnterLong(0, DefaultQuantity, "Buy");
				
				// Entry Conditions for Shorts
				else if (seasonalShort == true && lRSDXY > -lRSDXYSell && Closes[0][0] < MIN(Closes[0], Length)[1] - 2 && lRSNL < -lRSNLSell)
					EnterShort(0, DefaultQuantity, "Sell");
				
				// Exit Conditions for Longs
				if (Position.MarketPosition == MarketPosition.Long && BarsSinceEntry() > 1)
				{
					if (Closes[0][0] < EMA(Closes[0], mASell)[0] && lRSNL < lRSNLSell)
						ExitLong(0, DefaultQuantity, "L Exit RS", "Buy"); // RS with SNL exit
					
					else if (Closes[0][0] < EMA(Closes[0], mASell)[0] && EMA(Closes[1], mADXY)[0] > EMA(Closes[1], mADXY)[1] && lRSDXY > lRSDXYSell)
						ExitLong(0, DefaultQuantity, "L Exit DXY", "Buy"); // DXY Exit
				}
				
				// Exit Conditions for Shorts
				else if (Position.MarketPosition == MarketPosition.Short && BarsSinceEntry() > 1)
				{
					if (Closes[0][0] > EMA(Closes[0], mASell)[0] && lRSNL > -lRSNLSell)
						ExitShort(0, DefaultQuantity, "S Exit RS", "Sell");
					
					else if (Closes[0][0] > EMA(Closes[0], mASell)[0] && EMA(Closes[1], mADXY)[0] < EMA(Closes[1], mADXY)[1] && Closes[2][0] > EMA(Closes[2], mASNL)[0])
						ExitShort(0, DefaultQuantity, "S Exit DXY", "Sell"); // DXY Exit
				}
			}
        }

        #region Properties
        [Description("Channel length in days")]
        [Category("Parameters")]
        public int Length
        {
            get { return length; }
            set { length = Math.Max(1, value); }
        }

        [Description("Days for the Dollar Index linear regression slope calculation")]
        [Category("Parameters")]
        public int D1DXY
        {
            get { return d1DXY; }
            set { d1DXY = Math.Max(1, value); }
        }

        [Description("Days for the Seasonal data linear regression slope calculation")]
        [Category("Parameters")]
        public int D2SNL
        {
            get { return d2SNL; }
            set { d2SNL = Math.Max(1, value); }
        }

        [Description("Seasonal data linear regression slope per day")]
        [Category("Parameters")]
        public double LRSNLSell
        {
            get { return lRSNLSell; }
            set { lRSNLSell = value; }
        }

        [Description("Dollar index linear gression slope per day")]
        [Category("Parameters")]
        public double LRSDXYSell
        {
            get { return lRSDXYSell; }
            set { lRSDXYSell = value; }
        }

        [Description("Days for the seasonal exponential moving average sell/buytocover conditions")]
        [Category("Parameters")]
        public int MASNL
        {
            get { return mASNL; }
            set { mASNL = value; }
        }

        [Description("Days for the Soybean exponential moving average sell/buytocover conditions")]
        [Category("Parameters")]
        public int MASell
        {
            get { return mASell; }
            set { mASell = value; }
        }

        [Description("Days for the Dollar Index moving average sell/buytocover conditions")]
        [Category("Parameters")]
        public int MADXY
        {
            get { return mADXY; }
            set { mADXY = value; }
        }
		
		[Description("")]
        [Category("Parameters")]
        public int SellMonth
        {
            get { return sellMonth; }
            set { sellMonth = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int BuyMonth
        {
            get { return buyMonth; }
            set { buyMonth = Math.Max(1, value); }
        }
        #endregion
    }
}
